% Measures the minimum execution time of Volterra kernel calculation using
% SVD and BLAS functions.
%
% Discrete-time Volterra kernels of the cascade of 
% ideal discrete/continuous (D/C) converter with gain 1/Fs, 
% D/A reconstruction filter, loudspeaker A, A/D anti-aliasing filter and 
% ideal C/D converter.

% CBVK Toolbox
% JHMG, PMSB 2016


clear

m=11;       % Dimension
P=5;        % Expansion order
N=25;       % Memory length
    Fs=5000;    % Sampling frequency
rep=1;      % Number of repetitions for greater precision of execution
            % time measurement

arq1='CB_AfiltA_P5';                 % CB file            
arq2='mVK_svd_blas_AfiltA_P5_N25';   % Output file

%------------------------------------------


% Carleman Bilinearization 
load(arq1);

% Kernels

dt=zeros(1,rep);
for i=1:rep
    tic
    % Kernels of filters + loudspeaker
    kernels = vkrnsvdx(F,G,b,c,N,1/Fs,m,P);

    
    % Application of 1/Fs gain
    Ts=1/Fs;
    for p=1:P
        kernels{p}=kernels{p}*(Ts^p);
    end
    dt(i)=toc;
    fprintf('%g\n ', dt(i));
end
min(dt)
%save(arq2,'dt');


