% Carleman Bilinearization and Volterra Kernels Toolbox
%
% Jos Henrique de Morais Goulart (goulart@i3s.unice.fr)
% Phillip M. S. Burt (pmsburt@usp.br)
% 2016
%
% Main functions
% --------------
%
% Higher-level CB and Volterra kernels functions
%
% carlbl        - Computes the non-redundant Carleman Bilinearization of a linear-analytic system
% voltkernels   - Computes the triangular Volterra kernels of a linear-analytic system, using Carleman Bilinearization
% genmse        - Generates normalized MSE curves of a model with homogeneous components
% 
%
%
% Volterra Filter and related structures
%
% vfdiags   - Computes the output of a Volterra filter in the diagonal coordinate representation for a given input signal.
% vfilter   - Computes the output of a conventional Volterra filter for a given input signal.
% vcpd      - Computes the output of a Volterra-CPD model (also known as Volterra-PARAFAC model) for a given input
%
%
% Lower-level functions
%
% multaylor     - Computes the Taylor expansion of a multivariable vector function
% multaylorlit  - Computes the literal Taylor expansion of a multivariable vector function around the origin
% evtaylor      - Evaluates a literal Taylor expansion
% multaylorsp   - Literal Taylor expansion returned in a sparsity oriented format
% evtaylorsp    - Evaluates a sparsity oriented literal Taylor expansion 
%
% mtred, mtredlit, evtred, mtredsp, evtredsp   - Same as above but for the redundant Taylor expansion
%
% carlbilin     - Computes the non-redundant Carleman Bilinearization from the Taylor expansion
% carlbilinsp   - Same, using sparsity
% carlbidir     - Computes the direct Carleman Bilinearization from the redundant Taylor expansion
% carlbidirsp   - Same, using sparsity
%
% vkrn          - Computes triangular Volterra kernels from the Carleman Bilinearization of a system
% vkrnsvd       - Same, using SVD
% vkrnsvdx      - Same, using SVD and mex functions



