/* Selects non-redundant jacobian terms according to input vector
 
 Compile with mex selnrjac.c  
 
 
CBVK Toolbox
JHMG, PMSB 2016

*/

#include "mex.h"
#include "string.h"


void selnrjac(char *in0, size_t buflen, char *out, double *lini, int m)
{
  #define mMAX 20

  char *tt[mMAX], *t,*in;
  int i,j,N,linij;
  
  if (buflen == 0)
    return;

  /* Number of rows */
  N = (int) *(lini + m);
  

  /* Alocate memory string without blanks (worst case) */
  in = mxCalloc(buflen, sizeof(char));


  /* Alocate memory for element and segments (worst case) */
  t = mxCalloc(buflen, sizeof(char));
  for (j=0;j<m;j++){
    tt[j] = mxCalloc(buflen, sizeof(char));
  }

  
  /* Delete blanks */
  t = strtok(in0," ");	/* Initialize strtok */
  while (t != NULL){
    strcat(in,t);
    t = strtok(NULL," ");
  }

  /* Initialize strtok and copy 'matrix(' segment from input to output string */
  strcat(tt[0],strtok(in,"["));
  /* Add [[ to output */
  strcat(tt[0],"[[");

  /* Loop of rows of J (char(J) organizes J row-wise) */
  for (i=0;i<N-1;i++){

    /* Loop of columns of J */
    for (j=0;j<m;j++){

      /* Get an element of the Jacobian */
      t = strtok(NULL,",[]");


      if (i>= lini[j]){		/* Element is selected */
	strcat(tt[j],t);
	strcat(tt[j],",");
      }
    }
  }
  /* Last row */
  for (j=0;j<m-1;j++){
    t = strtok(NULL,",[]");
    strcat(tt[j],t);
    strcat(tt[j],",");
  }

  /* Last term */
    t = strtok(NULL,",[]");
    strcat(tt[m-1],t);
    strcat(tt[m-1],"]])");

  /* Concatenation of selected segments of each column */
    for (j=0;j<m;j++){
       strcat(out,tt[j]);
    }
    
}
    

void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[])
{
  char *input_buf, *output_buf;
    size_t buflen;
    double *lini;
    int m;
    
  
    /* Get the length of the input string */
    buflen = (mxGetM(prhs[0]) * mxGetN(prhs[0])) + 1;
    /* printf("%i\n",buflen); */

    /* Allocate memory for output string */
    output_buf = mxCalloc(buflen, sizeof(char));

    /* Copy the string data from prhs[0] into a C string input_ buf.    */
    input_buf = mxArrayToString(prhs[0]);

    /* Get initial rows vector and number of rows */
    lini = mxGetPr(prhs[1]);
    m = mxGetN(prhs[1])-1;

    /* Call the C subroutine */
    selnrjac(input_buf, buflen, output_buf, lini,m);
    
    /* Set C-style string output_buf to MATLAB mexFunction output*/
    plhs[0] = mxCreateString(output_buf);
    /* mxFree(input_buf); */
    return;
}

