/* Produces sparsity oriented representation
 
 Compile with mex spar.c  
 
 
CBVK Toolbox
JHMG, PMSB 2016

*/

#include "mex.h"

#include "string.h"


void spar(char *in0, size_t buflen, double *mp, char *fc, double *fi, int *N)
{
  int i;
  char *t, *in;
  
  if (buflen == 0) return;

  /* Alocate memory string without blanks (worst case) */
  in = mxCalloc(buflen, sizeof(char));


  /* Alocate memory for element and segments (worst case) */
  t = mxCalloc(buflen, sizeof(char));

  

  /* Delete blanks */
  t = strtok(in0," ");	/* Initialize strtok */
  while (t != NULL){
    strcat(in,t);
    t = strtok(NULL," ");
  }
  

  /* Adds [ to the output in order to multiply everything by the coefficient afterwards */
  strcat(fc,"[");

  *N = 0;			/* Non-null terms counter */
  i = mp[0] + 1;		/* Indices counter */
  t = strtok(in,",[]");		/* Initializes strtok */

  while ( (t != NULL) && (strcmp(t,"0")) == 0){ /* While null terms */
    i++;
    t = strtok(NULL,",[]");
  }

  if(t != NULL){
    fi[*N] = i;
    *N = *N + 1;
    strcat(fc,t);			/* First non-null term */
    t = strtok(NULL,",[]");
    i++;
    while (t != NULL){
      if (strcmp(t,"0") != 0){	/* Non-null term */
        strcat(fc,",");		/* Previous term wasn't the last one */
        fi[*N] = i;
        *N = *N + 1;
        strcat(fc,t);
        }
      t = strtok(NULL,",[]");
      i++;
    }
  }
  strcat(fc,"]");

}
    

void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[])
{
    char *input_buf, *fc_buf;
    size_t buflen;
    double *mp, *fit, *pf;
    int N=2,i;
    
  
    /* get the length of the input string */
    buflen = (mxGetM(prhs[0]) * mxGetN(prhs[0])) + 1;

    /* allocate memory for output string */
    fc_buf=mxCalloc(buflen, sizeof(char));

    /* copy the string data from prhs[0] into a C string input_ buf.    */
    input_buf = mxArrayToString(prhs[0]);

    /* get index vector */
    mp = mxGetPr(prhs[1]);

    /* create intermediate index vector (maximum length) */
    fit = mxCalloc(mp[1]-mp[0]+1,sizeof(double));


    /* call the C subroutine */
    spar(input_buf, buflen, mp, fc_buf, fit, &N);
    
    /* set C-style string output_buf to MATLAB mexFunction output*/
    plhs[0] = mxCreateString(fc_buf);

    /* create final index vector (necessary length N) */
    plhs[1] =  mxCreateDoubleMatrix(1,N,mxREAL);
    pf = mxGetPr(plhs[1]);
    /* fill final index vector */
    for(i=0;i<N;i++)
      pf[i] = fit[i];
   
    mxFree(input_buf);
    return;
}

