#include "mex.h"
#include "blas.h"

/* 
 
 Called by M-function VOLTKERNELS. Computes Volterra kernels 
  using pre-computed matrices obtained after order reduction 
 
 Compile with mex -largeArrayDims -lmwblas voltkrn.c  
 
 CBVK Toolbox
 JHMG, PMSB 2016

*/


void mexFunction( int nlhs, mxArray *plhs[],
        int nrhs, const mxArray *prhs[] )
{
    
    
    double *eU, *eV, *eUV, *pr, *pN, *pP, *h2, *h3, *h4, *h5,  m2, m3, m4, m5 ;
    double *Np;
    int r, N, N2, N3, P,rk;
    
    double *pv1, *pv1t, *pv2, *pv3, *pv4, *pt2, *pt3, *pt4, *peV1, *peV2, 
           *peV3, *peV4, *peUV1, *peUV2, *peUV3, *peUV4, *ph2, *ph3, *ph4, *ph5;
    int k1, k2, k3, k4, k5, i, j;
    
    mxArray *v2, *v3, *v4;
    
    
    /* Auxiliary BLAS variables */
    mwSignedIndex dim1, dim2, dim3, one=1;
    double res, oned=1, zerod=0;
    char *chnblas = "N";
    char *chtblas = "T";
    
    /* Pointer extraction and initialization for column-wise vectorization
     of input arguments. */
    
    eU = mxGetPr(prhs[0]);
    eV = mxGetPr(prhs[1]);
    eUV = mxGetPr(prhs[2]);
    pr = mxGetPr(prhs[3]);
    pN = mxGetPr(prhs[4]);
    pP = mxGetPr(prhs[5]);
    Np = mxGetPr(prhs[6]);
    
    r = (int) *pr;
    N = (int) *pN;
    P = (int) *pP;
    
    
    /* Output vector creation */
    plhs[0] = mxCreateDoubleMatrix((int)Np[0],1,mxREAL);
    h2 = mxGetPr(plhs[0]);
    if(P>2){
        plhs[1] = mxCreateDoubleMatrix((int)Np[1],1,mxREAL);
        h3 = mxGetPr(plhs[1]);
        if(P>3){
            plhs[2] = mxCreateDoubleMatrix((int)Np[2],1,mxREAL);
            h4 = mxGetPr(plhs[2]);
            if(P>4){
                plhs[3] = mxCreateDoubleMatrix((int)Np[3],1,mxREAL);
                h5 = mxGetPr(plhs[3]);
            }
        }
        
    }
    
    
    /* Intermediate vector creation */
    v2 = mxCreateDoubleMatrix(r,1,mxREAL);
    v3 = mxCreateDoubleMatrix(r,1,mxREAL);
    v4 = mxCreateDoubleMatrix(r,1,mxREAL);
    pv2 =mxGetPr(v2);
    pv3 =mxGetPr(v3);
    pv4 =mxGetPr(v4);
    
    /* Kernel pointer initialization */
    ph2 = h2; ph3 = h3; ph4 = h4; ph5 = h5;
    
    
    pv1 = eU;                   /* v1=eU{k1} */
    for(k1=0;k1<N;k1++){
        peV1 = eV;              /* peV1 points to first element of eV (k2=k1) */
        peUV1 = eUV;            /* peUV1 points to first element of eUV (k2=k1) */
                rk = 1;			/* Repetition counter for k indices */
        for(k2=k1;k2<N;k2++){
            
            if(k2==k1)
                rk += 1;
            else
                rk = 1;
            m2 = 1.0/rk;		/* Mask for h2 */
            
            /* ---------------------------------------------------- */
            /* h2(k2,k1)=v1*eV{k2-k1} */
            /* -----------------------------------------------------*/
            
            *ph2=0;
            pv1t = pv1;
            
            /* +++++++++++ BLAS dot product +++++++++++++++++++++++ */
             dim1 = r;
             *ph2 = ddot_(&dim1,pv1t,&one,peV1,&one);
             pv1t += r;         /* Pointer setting */
             peV1 += r;   
             /* +++++++++++ End of BLAS use +++++++++++++++++++++++ */
            
            /* peV1 points to next k2 in eV(k2-k1) */
            /* pv1t points to next k1 in eU(k1)    */
             
            *ph2 = *ph2 * m2;
            *ph2++;
            
            /* ---------------------------------------------------- */
            
            
            if(P>2){
                
                /* -----------------------------------------------------------------*/
                /*  v2 = v1*eUV{k2-k1+1}; */
                /* -----------------------------------------------------------------*/
                
                /* +++++++++++ BLAS vector-matrix product +++++++++++++++++++++++++ */
                dim1 = r;
                dim2 = r;
                dim3 = r;
                dgemv_(chtblas,&dim1,&dim2,&oned,peUV1,&dim3,pv1,&one,&zerod,pv2,&one);
                peUV1 += r*r;
                /* +++++++++++ End of BLAS use+ +++++++++++++++++++++++++++++++++++ */
                
                /* peUV1 points to next k2 int eUV(k2-k1) */
                
                /* ---------------------------------------------------------------- */
                
                       
             
                peV2 = eV;		/* peV2 points to first element of eV (k3=k2)   */
                peUV2 = eUV;	/* peUV2 points to first element of eUV (k3=k2) */
                
                for(k3=k2;k3<N;k3++){
                    
                    if(k3==k2)
                        rk += 1;
                    else
                        rk = 1;
                    m3 = m2/rk;  /* Mask for h3 */
                    
                    /* ------------------------------------------------------- */
                    /* h3(k1,k2,k3) = v2*eV{k3-k2+1};  */
                    /* ------------------------------------------------------- */
                    
                    /* +++++++++++ BLAS dot product ++++++++++++++++++++++++++ */
                     dim1 = r;
                     *ph3 = ddot_(&dim1,pv2,&one,peV2,&one);
                     peV2 += r;   
                    /* ++++++++++++ End of BLAS use+++++++++++++++++++++++++++ */                    
        
                    /* peV2 points to next k3 in eV(k3-k2) */
                            
                    *ph3 = *ph3 * m3;
                    *ph3++;                    
                        
                    /* ------------------------------------------------------- */
                    
                                       
                    if(P>3){
                    
                        /* ----------------------------------------------------------------- */
                        /* v3 = v2*eUV{k3-k2+1} */
                        /* ----------------------------------------------------------------- */
                        
                        /* +++++++++++ BLAS vector-matrix product ++++++++++++++++++++++++++ */
                        dim1 = r;
                        dim2 = r;
                        dim3 = r;
                        dgemv_(chtblas,&dim1,&dim2,&oned,peUV2,&dim3,pv2,&one,&zerod,pv3,&one);
                        peUV2 += r*r;
                        /* +++++++++++ End of BLAS use+ ++++++++++++++++++++++++++++++++++++ */                        
                        
                        /* peUV2 points to next k3 in eUV(k3-k2) */
                        
                        /* ------------------------------------------------------------------*/
                        
                        /* And so forth ... */
                        
                        peV3 = eV;          
                        peUV3 = eUV;        
                        for(k4=k3;k4<N;k4++){
                            
                            if(k4==k3)
                                rk += 1;
                            else
                                rk = 1;
                            m4 = m3/rk;       
                            
                            /* ------------------------------------------- */
                            /* h4(k1,k2,k3,k4) = v3*eV{k4-k3+1}; */
                            /* ------------------------------------------- */
                            
                             dim1 = r;
                             *ph4 = ddot_(&dim1,pv3,&one,peV3,&one);
                             peV3 += r;   
                            
                            *ph4 = *ph4 * m4;
                            *ph4++;
                            
                            /* ------------------------------------------- */
                            
                            if(P>4){
                                /* ----------------------------------------------------------------- */
                                /* v4 = v3*eUV{k4-k3+1} */
                                /* ----------------------------------------------------------------- */
                                dim1 = r;
                                dim2 = r;
                                dim3 = r;
                                dgemv_(chtblas,&dim1,&dim2,&oned,peUV3,&dim3,pv3,&one,&zerod,pv4,&one);
                                peUV3 += r*r;
                                /* ------------------------------------------------------------------*/
                                
                                
                                peV4 = eV;          
                                peUV4 = eUV;        
                                
                                for(k5=k4;k5<N;k5++){
                                    
                                    if(k5==k4)
                                        rk += 1;
                                    else
                                        rk = 1;
                                    m5 = m4/rk;                
                                    
                                    /*  ----------------------------------------------- */
                                    /* h5(k1,k2,k3,k4,k5) = v4*eV{k5-k4+1} */
                                    /* ------------------------------------------------ */
                                    
                                     dim1 = r;
                                     *ph5 = ddot_(&dim1,pv4,&one,peV4,&one);
                                     peV4 += r;   
                                                                        
                                    *ph5 = *ph5 * m5;
                                    *ph5++;
                                                                        
                                    /* ------------------------------------------------ */
                                }
                            }
                        }
                    }
                }
            }
        }
        pv1 = pv1t;   	/*  pt1 points to next k1 in v1=eU{k1}  */
    }
}

